searchData={"items":[{"type":"module","title":"P11ex.ECParam","doc":"This module provides functions to encode and decode elliptic curve parameters.","ref":"P11ex.ECParam.html"},{"type":"function","title":"P11ex.ECParam.ec_params_from_named_curve/1","doc":"Encode ECParameters for a named curve. ECParameters is a\nDER-encoded ASN.1 structure that identifies a named curve and\ncan be used as a value for the `cka_ec_params` attribute of\na key template.","ref":"P11ex.ECParam.html#ec_params_from_named_curve/1"},{"type":"function","title":"Examples - P11ex.ECParam.ec_params_from_named_curve/1","doc":"iex> P11ex.ECParam.ec_params_from_named_curve(:secp256r1)\n    {:ok, <<0x06, ...}\n\n    iex> P11ex.ECParam.ec_params_from_named_curve(:secp42r1)\n    {:error, \"Unknown named curve: secp42r1\"}","ref":"P11ex.ECParam.html#ec_params_from_named_curve/1-examples"},{"type":"function","title":"P11ex.ECParam.named_curves/0","doc":"List of named curves supported by the library.","ref":"P11ex.ECParam.html#named_curves/0"},{"type":"module","title":"P11ex.Flags","doc":"Handles conversion between PKCS#11 flags (CK_FLAGS) and MapSets of atoms.","ref":"P11ex.Flags.html"},{"type":"function","title":"P11ex.Flags.available_flags/1","doc":"Returns all possible flags for the given type.","ref":"P11ex.Flags.html#available_flags/1"},{"type":"function","title":"Examples - P11ex.Flags.available_flags/1","doc":"iex> P11ex.Flags.available_flags(:slot)\n    [:hw_slot, :removable_device, :token_present]","ref":"P11ex.Flags.html#available_flags/1-examples"},{"type":"type","title":"P11ex.Flags.flag_name/0","doc":"","ref":"P11ex.Flags.html#t:flag_name/0"},{"type":"type","title":"P11ex.Flags.flag_type/0","doc":"","ref":"P11ex.Flags.html#t:flag_type/0"},{"type":"type","title":"P11ex.Flags.flag_value/0","doc":"","ref":"P11ex.Flags.html#t:flag_value/0"},{"type":"function","title":"P11ex.Flags.from_atoms/2","doc":"Converts a MapSet of atoms to a flags integer for the given flag type.","ref":"P11ex.Flags.html#from_atoms/2"},{"type":"function","title":"Examples - P11ex.Flags.from_atoms/2","doc":"iex> P11ex.Flags.from_atoms(:slot, MapSet.new([:hw_slot, :removable_device]))\n    0x0003","ref":"P11ex.Flags.html#from_atoms/2-examples"},{"type":"function","title":"P11ex.Flags.to_atoms/2","doc":"Converts a flags integer to a MapSet of atoms for the given flag type.","ref":"P11ex.Flags.html#to_atoms/2"},{"type":"function","title":"Examples - P11ex.Flags.to_atoms/2","doc":"iex> P11ex.Flags.to_atoms(:slot, 0x0003)\n    #MapSet<[:hw_slot, :removable_device]>","ref":"P11ex.Flags.html#to_atoms/2-examples"},{"type":"module","title":"P11ex.Lib","doc":"This module contains the core functionality for the `P11ex` library and provides\nthe low-level API for interacting with PKCS#11 modules. In general, you should not\nuse this module directly. Instead, use the higher-level `P11ex.Module` and `P11ex.Session`\nmodules instead.","ref":"P11ex.Lib.html"},{"type":"type","title":"P11ex.Lib.attribute/0","doc":"","ref":"P11ex.Lib.html#t:attribute/0"},{"type":"type","title":"P11ex.Lib.attributes/0","doc":"","ref":"P11ex.Lib.html#t:attributes/0"},{"type":"function","title":"P11ex.Lib.close_all_sessions/2","doc":"","ref":"P11ex.Lib.html#close_all_sessions/2"},{"type":"function","title":"P11ex.Lib.close_session/1","doc":"","ref":"P11ex.Lib.html#close_session/1"},{"type":"function","title":"P11ex.Lib.decrypt/4","doc":"Decrypt data using the specified `mechanism` and `key` in a single call. See\n`P11ex.Lib.encrypt_init/3` on how to select a decryption mechanism and\nset its parameters. Consider using `P11ex.Lib.decrypt_init/3`,\n`P11ex.Lib.decrypt_update/2`, and `P11ex.Lib.decrypt_final/1` if you\nwant to decrypt data in chunks.","ref":"P11ex.Lib.html#decrypt/4"},{"type":"function","title":"P11ex.Lib.decrypt_final/1","doc":"Finalize the decryption operation that is in progress for this session\n(see `P11ex.Lib.decrypt_init/3`).","ref":"P11ex.Lib.html#decrypt_final/1"},{"type":"function","title":"P11ex.Lib.decrypt_init/3","doc":"Initialize a decryption operation involving the specified `mechanism` and `key`.\nMany mechanisms require additional parameters. See `P11ex.Lib.encrypt_init/3` for more\ninformation on mechanisms and their parameters.\n\nThe function returns `:ok` if the operation is initialized successfully. The session\nis now in decryption mode and no other operations can be active at the same time. The\nciphertext can be provided in chunks using `P11ex.Lib.decrypt_update/2` and\n`P11ex.Lib.decrypt_final/1`.","ref":"P11ex.Lib.html#decrypt_init/3"},{"type":"function","title":"P11ex.Lib.decrypt_update/2","doc":"Provide a chunk of ciphertext data to the decryption operation that is in\nprogress for this session (see `P11ex.Lib.decrypt_init/3`).","ref":"P11ex.Lib.html#decrypt_update/2"},{"type":"function","title":"P11ex.Lib.destroy_object/2","doc":"","ref":"P11ex.Lib.html#destroy_object/2"},{"type":"function","title":"P11ex.Lib.digest/2","doc":"Get the digest of the data provided to the digest operation. The session must\nbe in the `:digest` state, so this function must be called after `digest_init/2`.\nUse `digest/2` to provide all data at once and get the digest in one go.","ref":"P11ex.Lib.html#digest/2"},{"type":"function","title":"P11ex.Lib.digest_final/1","doc":"Finalize the digest operation. The session must be in the `:digest` state,\nso this function must be called after `digest_init/2` and `digest_update/2`.\nIf the operation fails, the session's current operation is reset. The function\nreturns the digest.","ref":"P11ex.Lib.html#digest_final/1"},{"type":"function","title":"P11ex.Lib.digest_init/2","doc":"Initialize a digest operation. The session's current operation is set to\n`:digest`. Use `digest_update/2` to provide data to the digest operation.\nCall `digest_final/1` to finalize the operation and get the digest. Or, call\n`digest/2` to provide all data at once and get the digest in one go.","ref":"P11ex.Lib.html#digest_init/2"},{"type":"function","title":"Example: Digest computation in chunks - P11ex.Lib.digest_init/2","doc":"```elixir\n:ok = P11ex.Session.digest_init(session, {:ckm_sha256})\n:ok = P11ex.Session.digest_update(session, data1)\n:ok = P11ex.Session.digest_update(session, data2)\n{:ok, digest} = P11ex.Session.digest_final(session)\n```","ref":"P11ex.Lib.html#digest_init/2-example-digest-computation-in-chunks"},{"type":"function","title":"Example: Digest computation in one go - P11ex.Lib.digest_init/2","doc":"```elixir\n:ok = P11ex.Session.digest_init(session, {:ckm_sha256})\n{:ok, digest} = P11ex.Session.digest(session, data)\n```","ref":"P11ex.Lib.html#digest_init/2-example-digest-computation-in-one-go"},{"type":"function","title":"P11ex.Lib.digest_update/2","doc":"Provide data to the digest operation. The session must be in the `:digest`\nstate, so this function must be called after `digest_init/2`. Call this\nfunction repeatedly with chunks of data until all data has been provided.\nIf the operation fails, the session's current operation is reset.","ref":"P11ex.Lib.html#digest_update/2"},{"type":"function","title":"P11ex.Lib.encrypt/4","doc":"Encrypt data using the specified `mechanism` and `key` in a single call. Consider\nusing `P11ex.Lib.encrypt_init/3`, `P11ex.Lib.encrypt_update/2`, and\n`P11ex.Lib.encrypt_final/1` if you want to encrypt data in chunks.\n\nSee `P11ex.Lib.encrypt_init/3` for examples on how to select an encryption mechanism\nand set its parameters.","ref":"P11ex.Lib.html#encrypt/4"},{"type":"function","title":"P11ex.Lib.encrypt_final/1","doc":"Finalize the encryption operation that is in progress for this session.","ref":"P11ex.Lib.html#encrypt_final/1"},{"type":"function","title":"P11ex.Lib.encrypt_init/3","doc":"Initialize an encryption operation involving the specified `mechanism` and `key`. This puts\nthe session into encryption mode and no other operations can be active at the same time. Use\n`encrypt_update/2` and `encrypt_final/1` to provide data to encrypt and produce the ciphertext\nchunks. Consider using `encrypt/4` if you want to encrypt data in a single call.","ref":"P11ex.Lib.html#encrypt_init/3"},{"type":"function","title":"Setting an encryption mechanism - P11ex.Lib.encrypt_init/3","doc":"Some mechanisms require additional parameters. These parameters are passed as a\nmap. The NIF will translate the map into the appropriate PKCS#11 mechanism structure.\nIf this translation fails (e.g. missing required parameters or wrong type) the operation\nwill return an error of the form `{:error, :invalid_parameter, reason}`.\n\nThe following example show how to do this for common mechanisms:","ref":"P11ex.Lib.html#encrypt_init/3-setting-an-encryption-mechanism"},{"type":"function","title":"AES ECB - P11ex.Lib.encrypt_init/3","doc":"No additional parameters are required for AES ECB mode.\n\n```elixir\n:ok = P11ex.Session.encrypt_init(session, {:ckm_aes_ecb}, key)\n```","ref":"P11ex.Lib.html#encrypt_init/3-aes-ecb"},{"type":"function","title":"AES CBC and AES OFB - P11ex.Lib.encrypt_init/3","doc":"These mechanisms require an initialization vector (IV). This IV has to be the same length\nas the block size of the cipher. For AES the block size is 16 bytes and thus the IV\nhas to be 16 bytes long.\n\n```elixir\niv = :crypto.strong_rand_bytes(16)\n:ok = P11ex.Session.encrypt_init(session1, {:ckm_aes_cbc, %{iv: iv}}, key)\n:ok = P11ex.Session.encrypt_init(session2, {:ckm_aes_ofb, %{iv: iv}}, key)\n```","ref":"P11ex.Lib.html#encrypt_init/3-aes-cbc-and-aes-ofb"},{"type":"function","title":"AES CTR - P11ex.Lib.encrypt_init/3","doc":"This mechanism requires an initialization vector (IV) and the number of bits in the counter\n(e.g. 32, 64, 128).\n\n```elixir\niv = :crypto.strong_rand_bytes(16)\nparams = %{iv: iv, counter_bits: 32}\n:ok = P11ex.Session.encrypt_init(session, {:ckm_aes_ctr, params}, key)\n```","ref":"P11ex.Lib.html#encrypt_init/3-aes-ctr"},{"type":"function","title":"AES GCM - P11ex.Lib.encrypt_init/3","doc":"This mechanism has the following additional parameters:\n* `:iv` - the initialization vector (IV). Typically, this is 12 bytes long.\n* `:aad` - the optional authentication data (AAD). Not all PKCS#11 tokens support this parameter.\n  Also, the size of the AAD is limited by the token.\n* `:tag_bits` - the number of bits in the authentication tag (typically 128)\n\n```elixir\niv = :crypto.strong_rand_bytes(12)\nparams = %{iv: iv, tag_bits: 128}\n:ok = P11ex.Session.encrypt_init(session, {:ckm_aes_gcm, params}, key)\n```","ref":"P11ex.Lib.html#encrypt_init/3-aes-gcm"},{"type":"function","title":"RSA with PKCS#1 v1.5 - P11ex.Lib.encrypt_init/3","doc":"This mechanism requires a RSA public key and does not require any additional parameters.\n\n```elixir\n:ok = P11ex.Session.encrypt_init(session, {:ckm_rsa_pkcs}, pub_key)\n{:ok, ciphertext} = P11ex.Session.encrypt(session, data)\n```","ref":"P11ex.Lib.html#encrypt_init/3-rsa-with-pkcs-1-v1-5"},{"type":"function","title":"RSA OAEP - P11ex.Lib.encrypt_init/3","doc":"This mechanism requires a RSA public key and the following parameters:\n\n- `:hash_alg` - the hash algorithm to use.\n- `:mgf_hash_alg` - the hash algorithm to use for the mask generation function.\n- `:source_data` - the source data to use for the OAEP padding.\n\nThe `hash_alg` and `mgf_hash_alg` parameters identify an hash algorithm in the\nsame way as the `:crypto` module does. That is, possible values are `:sha`,\n`:sha224`, `:sha256`, `:sha384`, and `:sha512`. Support depends on the token.\n\nExample:\n\n```elixir\n:ok = P11ex.Session.encrypt_init(session, {:ckm_rsa_pkcs_oaep, %{hash_alg: :sha, mgf_hash_alg: :sha, source_data: source_data}}, pub_key)\n{:ok, ciphertext} = P11ex.Session.encrypt(session, data)\n```","ref":"P11ex.Lib.html#encrypt_init/3-rsa-oaep"},{"type":"function","title":"P11ex.Lib.encrypt_update/2","doc":"Provide a chunk of plaintext data to the encryption operation that is in\nprogress for this session (see `P11ex.Lib.encrypt_init/3`).","ref":"P11ex.Lib.html#encrypt_update/2"},{"type":"function","title":"P11ex.Lib.finalize/1","doc":"","ref":"P11ex.Lib.html#finalize/1"},{"type":"function","title":"P11ex.Lib.find_objects/3","doc":"","ref":"P11ex.Lib.html#find_objects/3"},{"type":"function","title":"P11ex.Lib.generate_key/3","doc":"Generate a symmetric key in the session. The key is generated according to the specified `mechanism`\nand the `key_template`. The `key_template` is a list of attributes that will be used to generate\nthe key. The function returns a handle to the generated key.","ref":"P11ex.Lib.html#generate_key/3"},{"type":"function","title":"Example: Generate a 128-bit AES key - P11ex.Lib.generate_key/3","doc":"The following example generates a 128-bit AES key with the label \"test_key\" and a random\nkey ID. The key is a session key.\n\n```elixir\nkey_id = :crypto.strong_rand_bytes(16)\n{:ok, key} =\n  Session.generate_key(context.session_pid,\n    {:ckm_aes_key_gen},\n    [\n      {:cka_token, false},\n      {:cka_label, \"test_key\"},\n      {:cka_value_len, key_id},\n      {:cka_id, key_id},\n      {:cka_encrypt, true},\n      {:cka_decrypt, false},\n      {:cka_derive, false},\n      {:cka_sign, false}\n    ])\n```","ref":"P11ex.Lib.html#generate_key/3-example-generate-a-128-bit-aes-key"},{"type":"function","title":"P11ex.Lib.generate_key_pair/4","doc":"Generate a key pair in the session. The key pair is generated according to the specified `mechanism`\nand the `pub_key_template` and `priv_key_template`. The function returns a tuple with the public\nand private key handles.","ref":"P11ex.Lib.html#generate_key_pair/4"},{"type":"function","title":"Example: Generate a RSA key pair - P11ex.Lib.generate_key_pair/4","doc":"```elixir\nmechanism = {:ckm_rsa_pkcs_key_pair_gen}\n\npubk_template = [\n  {:cka_token, false},\n  {:cka_encrypt, true},\n  {:cka_verify, true},\n  {:cka_modulus_bits, 2048},\n  {:cka_public_exponent, 65537},\n  {:cka_label, \"rsa_test_key\"}\n]\n\nprvk_template = [\n  {:cka_token, false},\n  {:cka_private, true},\n  {:cka_sensitive, true},\n  {:cka_decrypt, true},\n  {:cka_sign, true},\n  {:cka_label, \"rsa_test_key\"}\n]\n\n{pubk, prvk} =\n  P11ex.Session.generate_key_pair(session_pid,\n  {:ckm_rsa_pkcs_key_pair_gen},\n  pubk_template, prvk_template)\n```","ref":"P11ex.Lib.html#generate_key_pair/4-example-generate-a-rsa-key-pair"},{"type":"function","title":"Example: Generate an EC key pair (secp256r1) - P11ex.Lib.generate_key_pair/4","doc":"See `P11ex.ECParam.ec_params_from_named_curve/1` for more functions that\nhelp to create the value of the `:cka_ec_params` attribute.\n\n```elixir\nkey_id = :crypto.strong_rand_bytes(16)\n\nmechanism = {:ckm_ec_key_pair_gen}\n{:ok, params} = ECParam.ec_params_from_named_curve(:secp256r1)\n\npubk_template = [\n  {:cka_token, false},\n  {:cka_key_type, :ckk_ec},\n  {:cka_verify, true},\n  {:cka_label, \"pubk-secp256r1\"},\n  {:cka_ec_params, params},\n  {:cka_id, key_id}\n]\n\nprvk_template = [\n  {:cka_token, false},\n  {:cka_key_type, :ckk_ec},\n  {:cka_sign, true},\n  {:cka_label, \"prvk-secp256r1\"},\n  {:cka_id, key_id}\n]\n\n{:ok, {pubk, prvk}} =\n    Session.generate_key_pair(context.session_pid,\n        mechanism, pubk_template, prvk_template)\n```","ref":"P11ex.Lib.html#generate_key_pair/4-example-generate-an-ec-key-pair-secp256r1"},{"type":"function","title":"P11ex.Lib.generate_random/2","doc":"Generate random data using the token's RNG.","ref":"P11ex.Lib.html#generate_random/2"},{"type":"function","title":"P11ex.Lib.get_object_attributes/3","doc":"","ref":"P11ex.Lib.html#get_object_attributes/3"},{"type":"function","title":"P11ex.Lib.list_mechanisms/2","doc":"","ref":"P11ex.Lib.html#list_mechanisms/2"},{"type":"function","title":"P11ex.Lib.list_slots/2","doc":"","ref":"P11ex.Lib.html#list_slots/2"},{"type":"function","title":"P11ex.Lib.load_module/1","doc":"","ref":"P11ex.Lib.html#load_module/1"},{"type":"function","title":"P11ex.Lib.load_nifs/0","doc":"","ref":"P11ex.Lib.html#load_nifs/0"},{"type":"function","title":"P11ex.Lib.mechanism_info/3","doc":"","ref":"P11ex.Lib.html#mechanism_info/3"},{"type":"type","title":"P11ex.Lib.mechanism_instance/0","doc":"A mechanism instance represents a cryptographic mechanism with the\nassociated parameters. A mechanism can either be identified by an\natom (e.g. `:aes_cbc`) or a non-negative integer (e.g. `1`). Some\nmechanisms require additional parameters that are passed as a map.\n\nExample:\n```elixir\n{:ckm_aes_cbc, %{iv: iv}}\n```\nThis is AES in CBC mode with the initialization vector `iv`\n(a binary of 16 bytes).\n\nSee `P11ex.Session.encrypt_init/3` for examples on how to set the\nparameters for the various encryption mechanisms.","ref":"P11ex.Lib.html#t:mechanism_instance/0"},{"type":"function","title":"P11ex.Lib.open_session/3","doc":"","ref":"P11ex.Lib.html#open_session/3"},{"type":"function","title":"P11ex.Lib.session_info/1","doc":"","ref":"P11ex.Lib.html#session_info/1"},{"type":"function","title":"P11ex.Lib.session_login/3","doc":"","ref":"P11ex.Lib.html#session_login/3"},{"type":"function","title":"P11ex.Lib.session_logout/1","doc":"","ref":"P11ex.Lib.html#session_logout/1"},{"type":"function","title":"P11ex.Lib.sign/2","doc":"Sign or MAC data. The session must be in the `:sign` state, so this function\nmust be called after `sign_init/3`. If the operation\nfails, the session's current operation is reset. The function returns the\nsignature or MAC.","ref":"P11ex.Lib.html#sign/2"},{"type":"function","title":"P11ex.Lib.sign_final/1","doc":"Finalize the signing operation or MAC computation. The session must\nbe in the `:sign` state, so this function must be called after\n`sign_init/3` and `sign_update/2`. If the operation fails, the session's\ncurrent operation is reset. The function returns the signature or MAC.","ref":"P11ex.Lib.html#sign_final/1"},{"type":"function","title":"P11ex.Lib.sign_init/3","doc":"Initialize a signing operation or MAC computation. The `key`'s type\nmust be suitable for the specified `mechanism`. If the initialization\nis successful, the session's current operation is set to `:sign`. This\noperation can be finalized by calling `sign_final/1` or `sign/2`. Use\n`sign_update/2` to provide data to the signing operation.","ref":"P11ex.Lib.html#sign_init/3"},{"type":"function","title":"Example: Signing data in chunks - P11ex.Lib.sign_init/3","doc":"```elixir\n:ok = Session.sign_init(session, {:ckm_rsa_pkcs, %{hash_alg: :sha256}}, priv_key)\n:ok = Session.sign_update(session, data1)\n:ok = Session.sign_update(session, data2)\n{:ok, signature} = Session.sign_final(session)\n```","ref":"P11ex.Lib.html#sign_init/3-example-signing-data-in-chunks"},{"type":"function","title":"Example: Signing data in one go - P11ex.Lib.sign_init/3","doc":"```elixir\n:ok = Session.sign_init(session, {:ckm_rsa_pkcs, %{hash_alg: :sha256}}, priv_key)\n{:ok, signature} = Session.sign(session, data)\n```","ref":"P11ex.Lib.html#sign_init/3-example-signing-data-in-one-go"},{"type":"function","title":"Signing Mechanisms - P11ex.Lib.sign_init/3","doc":"","ref":"P11ex.Lib.html#sign_init/3-signing-mechanisms"},{"type":"function","title":"RSA PKCS #1 v1.5 Signature and Encryption Mechanism - P11ex.Lib.sign_init/3","doc":"This mechanism requires a RSA private key and does not require any\nadditional parameters. The digest algorithm to use is specified in the\nmechanism name. The following mechanisms fall into this category:\n\n- `:ckm_rsa_pkcs` (uses plain RSA PKCS#1 v1.5 without digest computation)\n- `:ckm_sha1_rsa_pkcs`\n- `:ckm_sha224_rsa_pkcs`\n- `:ckm_sha256_rsa_pkcs`\n- `:ckm_sha384_rsa_pkcs`\n- `:ckm_sha512_rsa_pkcs`\n\nExample:\n\n```elixir\n:ok = Session.sign_init(session, {:ckm_sha256_rsa_pkcs}, priv_key)\n{:ok, signature} = Session.sign(session, data)\n```","ref":"P11ex.Lib.html#sign_init/3-rsa-pkcs-1-v1-5-signature-and-encryption-mechanism"},{"type":"function","title":"RSA PKCS #1 PSS Signature Mechanism (:ckm_rsa_pkcs_pss) - P11ex.Lib.sign_init/3","doc":"This mechanism requires a RSA private key and the following parameters:\n\n- `:salt_len` - the length of the salt in bytes.\n- `:hash_alg` - the hash algorithm to use.\n- `:mgf_hash_alg` - the hash algorithm to use for the mask generation function.\n\nThe `hash_alg` and `mgf_hash_alg` parameters identify an hash algorithm in the\nsame way as the `:crypto` module does. That is, possible values are `:sha`,\n`:sha224`, `:sha256`, `:sha384`, and `:sha512`.\n\nExample:\n\n```elixir\n:ok = Session.sign_init(session, {:ckm_rsa_pkcs_pss, %{salt_len: 20, hash_alg: :sha256, mgf_hash_alg: :sha256}}, priv_key)\n{:ok, signature} = Session.sign(session, data)\n```","ref":"P11ex.Lib.html#sign_init/3-rsa-pkcs-1-pss-signature-mechanism-ckm_rsa_pkcs_pss"},{"type":"function","title":"ECDSA Signature Mechanism (:ckm_ecdsa) - P11ex.Lib.sign_init/3","doc":"This algorithm requires a pre-computed digest of the data to sign. That is,\nit does not compute the digest itself.\n\nExample:\n\n```elixir\ndata = :crypto.strong_rand_bytes(1024)\ndigest = :crypto.hash(:sha256, data)\n\n:ok = Session.sign_init(session, {:ckm_ecdsa}, priv_key)\n{:ok, signature} = Session.sign(session, digest)\n```","ref":"P11ex.Lib.html#sign_init/3-ecdsa-signature-mechanism-ckm_ecdsa"},{"type":"function","title":"P11ex.Lib.sign_update/2","doc":"Provide data to the signing operation or MAC computation. The session must\nbe in the `:sign` state, so this function must be called after `sign_init/3`.\nCall this function repeatedly with chunks of data until all data has been\nprovided. If the operation fails, the session's current operation is reset.","ref":"P11ex.Lib.html#sign_update/2"},{"type":"function","title":"P11ex.Lib.token_info/2","doc":"","ref":"P11ex.Lib.html#token_info/2"},{"type":"function","title":"P11ex.Lib.verify/3","doc":"Verify a signature or MAC. The session must be in the `:verify` state, so this function\nmust be called after `verify_init/3`. If the operation fails, the session's current\noperation is reset.\n\nThe operation return `:ok` if the signature (or MAC) is valid. Otherwise, it returns\nan error. Typically, the error reason is `:ckr_signature_invalid` or `:ckr_signature_len_range`.","ref":"P11ex.Lib.html#verify/3"},{"type":"function","title":"P11ex.Lib.verify_init/3","doc":"Initialize a verification operation involving the specified `mechanism` and `key`.\nThe operation verifies signatures or MACs, depending the mechanism. Some mechanisms\nrequire additional parameters. See `P11ex.Lib.sign_init/3` for more information\non mechanisms and their parameters.\n\nIf successful, the session is in verification mode and no other operations can be\nactive at the same time.","ref":"P11ex.Lib.html#verify_init/3"},{"type":"function","title":"Example: Verifying a RSA PKCS #1 v1.5 signature - P11ex.Lib.verify_init/3","doc":"```elixir\n:ok = Session.verify_init(session, {:ckm_sha256_rsa_pkcs}, pub_key)\n:ok = Session.verify(session, data, signature)\n```","ref":"P11ex.Lib.html#verify_init/3-example-verifying-a-rsa-pkcs-1-v1-5-signature"},{"type":"module","title":"P11ex.Lib.ModuleHandle","doc":"Represents a reference to a dynamically loaded PKCS#11 module.","ref":"P11ex.Lib.ModuleHandle.html"},{"type":"type","title":"P11ex.Lib.ModuleHandle.path/0","doc":"The path to the shared PKCS#11 library module file. Must always be a valid string.","ref":"P11ex.Lib.ModuleHandle.html#t:path/0"},{"type":"type","title":"P11ex.Lib.ModuleHandle.ref/0","doc":"NIF reference to the loaded PKCS#11 module.","ref":"P11ex.Lib.ModuleHandle.html#t:ref/0"},{"type":"type","title":"P11ex.Lib.ModuleHandle.t/0","doc":"A struct representing a loaded PKCS#11 module.","ref":"P11ex.Lib.ModuleHandle.html#t:t/0"},{"type":"type","title":"Fields: - P11ex.Lib.ModuleHandle.t/0","doc":"- `path` (String.t()): The file path of the module (always required).\n  - `ref` (reference()): A NIF reference to the loaded module.","ref":"P11ex.Lib.ModuleHandle.html#t:t/0-fields"},{"type":"module","title":"P11ex.Lib.ObjectAttributes","doc":"This module defines sets of attributes for PKCS#11 objects.","ref":"P11ex.Lib.ObjectAttributes.html"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.common/0","doc":"Attributes that all kinds of objects have.","ref":"P11ex.Lib.ObjectAttributes.html#common/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.ec_private_key/0","doc":"Attributes that can be found on EC private keys.","ref":"P11ex.Lib.ObjectAttributes.html#ec_private_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.ec_public_key/0","doc":"Attributes that can be found on EC public keys.","ref":"P11ex.Lib.ObjectAttributes.html#ec_public_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.key/0","doc":"Attributes related to keys. This are attributes can be found on secrets keys,\npublic keys, and private keys.","ref":"P11ex.Lib.ObjectAttributes.html#key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.private_key/0","doc":"Attributes that can be found on private keys.","ref":"P11ex.Lib.ObjectAttributes.html#private_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.public_key/0","doc":"Attributes that can be found on public keys.","ref":"P11ex.Lib.ObjectAttributes.html#public_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.rsa_private_key/0","doc":"Attributes that can be found on RSA private keys.","ref":"P11ex.Lib.ObjectAttributes.html#rsa_private_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.rsa_private_key_with_sensitive/0","doc":"Attributes that can be found on RSA private keys that are sensitive. The token will\nnot return these attributes unless the `:cka_sensitive` attribute is set to `false`\nor `:cka_extractable` is set to `true`.","ref":"P11ex.Lib.ObjectAttributes.html#rsa_private_key_with_sensitive/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.rsa_public_key/0","doc":"Attributes that can be found on RSA public keys.","ref":"P11ex.Lib.ObjectAttributes.html#rsa_public_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.secret_key/0","doc":"Attributes that can be found on secret keys.","ref":"P11ex.Lib.ObjectAttributes.html#secret_key/0"},{"type":"function","title":"P11ex.Lib.ObjectAttributes.storage/0","doc":"Attributes related to the storage of objects. Most objects have these attributes.","ref":"P11ex.Lib.ObjectAttributes.html#storage/0"},{"type":"module","title":"P11ex.Lib.ObjectHandle","doc":"Represents a PKCS#11 object. This can be a key, a certificate, a secret key, etc. Note\nthat the object handle may be only valid in the context of the session that created it.\nFor example, a session key (`:cka_token` is `false`) is only visible and usable within the\ncontext of the session that generates it. Other handles may be visible and usable over multiple\nsessions, such as handles to token objects.","ref":"P11ex.Lib.ObjectHandle.html"},{"type":"type","title":"P11ex.Lib.ObjectHandle.handle/0","doc":"The handle of the object which is unsigned integer identifying the object.","ref":"P11ex.Lib.ObjectHandle.html#t:handle/0"},{"type":"function","title":"P11ex.Lib.ObjectHandle.new/1","doc":"Create a new object handle and do not associate it with a session.","ref":"P11ex.Lib.ObjectHandle.html#new/1"},{"type":"function","title":"P11ex.Lib.ObjectHandle.new/2","doc":"Create a new object handle and associate it with a session.","ref":"P11ex.Lib.ObjectHandle.html#new/2"},{"type":"type","title":"P11ex.Lib.ObjectHandle.session/0","doc":"The PKCS#11 session that the object belongs to. May be `nil` if the is not known\nwhich session the object belongs to.","ref":"P11ex.Lib.ObjectHandle.html#t:session/0"},{"type":"type","title":"P11ex.Lib.ObjectHandle.t/0","doc":"A struct representing a PKCS#11 object.","ref":"P11ex.Lib.ObjectHandle.html#t:t/0"},{"type":"module","title":"P11ex.Lib.SessionHandle","doc":"Represents a PKCS#11 session. A session is used to interact with a token.","ref":"P11ex.Lib.SessionHandle.html"},{"type":"type","title":"P11ex.Lib.SessionHandle.handle/0","doc":"The handle of the session.","ref":"P11ex.Lib.SessionHandle.html#t:handle/0"},{"type":"function","title":"P11ex.Lib.SessionHandle.new/3","doc":"Create a new session handle.","ref":"P11ex.Lib.SessionHandle.html#new/3"},{"type":"type","title":"P11ex.Lib.SessionHandle.pkcs11_module/0","doc":"The PKCS#11 module that the session belongs to.","ref":"P11ex.Lib.SessionHandle.html#t:pkcs11_module/0"},{"type":"type","title":"P11ex.Lib.SessionHandle.slot_id/0","doc":"The slot identifier of the session.","ref":"P11ex.Lib.SessionHandle.html#t:slot_id/0"},{"type":"type","title":"P11ex.Lib.SessionHandle.t/0","doc":"A struct representing a PKCS#11 session.","ref":"P11ex.Lib.SessionHandle.html#t:t/0"},{"type":"module","title":"P11ex.Lib.Slot","doc":"Represents a PKCS#11 slot. A slot can contain a token (e.g. a smart card) or a token emulator\n(e.g. a software token).","ref":"P11ex.Lib.Slot.html"},{"type":"type","title":"P11ex.Lib.Slot.description/0","doc":"The slot description.","ref":"P11ex.Lib.Slot.html#t:description/0"},{"type":"type","title":"P11ex.Lib.Slot.firmware_version/0","doc":"The firmware version of the slot.","ref":"P11ex.Lib.Slot.html#t:firmware_version/0"},{"type":"type","title":"P11ex.Lib.Slot.flags/0","doc":"The flags of the slot. See `P11ex.Flags` for more information.","ref":"P11ex.Lib.Slot.html#t:flags/0"},{"type":"type","title":"P11ex.Lib.Slot.hardware_version/0","doc":"The hardware version of the slot.","ref":"P11ex.Lib.Slot.html#t:hardware_version/0"},{"type":"type","title":"P11ex.Lib.Slot.manufacturer_id/0","doc":"The manufacturer ID of the slot.","ref":"P11ex.Lib.Slot.html#t:manufacturer_id/0"},{"type":"type","title":"P11ex.Lib.Slot.pkcs11_module/0","doc":"The PKCS#11 module that the slot belongs to.","ref":"P11ex.Lib.Slot.html#t:pkcs11_module/0"},{"type":"type","title":"P11ex.Lib.Slot.slot_id/0","doc":"The slot identifier.","ref":"P11ex.Lib.Slot.html#t:slot_id/0"},{"type":"type","title":"P11ex.Lib.Slot.t/0","doc":"A struct representing a PKCS#11 slot.","ref":"P11ex.Lib.Slot.html#t:t/0"},{"type":"module","title":"P11ex.Module","doc":"A module is a `GenServer` that manages a PKCS#11 module and its loading state. A PKCS#11 module is a\nshared library that implements a PKCS#11 provider.  A module should be loaded only once per application\nor beam virtual machine. That is, you should only create one instance of `P11ex.Module` in your application\nand add it to your supervision tree. Operations on the module should be performed through the `GenServer`\ncallbacks so that they are serialized.","ref":"P11ex.Module.html"},{"type":"module","title":"Loading a module - P11ex.Module","doc":"To load a module, you can use the `start_link/1` function. The argument is the path to the module file.\nThe module will be loaded and initialized.\n\n```elixir\ndefmodule MyApp.Supervisor do\n  use Supervisor\n\n  def start_link(init_arg) do\n    Supervisor.start_link(__MODULE__, init_arg, name: __MODULE__)\n  end\n\n  def init(init_arg) do\n    children = [\n      {P11ex.Module, \"/usr/lib/softhsm/libsofthsm2.so\"}\n    ]\n    Supervisor.init(children, strategy: :one_for_one)\n  end\nend\n```","ref":"P11ex.Module.html#module-loading-a-module"},{"type":"function","title":"P11ex.Module.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"P11ex.Module.html#child_spec/1"},{"type":"function","title":"P11ex.Module.find_slot_by_tokenlabel/1","doc":"Find the slot that contains a token with the given label.","ref":"P11ex.Module.html#find_slot_by_tokenlabel/1"},{"type":"function","title":"P11ex.Module.list_mechanisms/1","doc":"List all mechanisms supported by the PKCS#11 module for a slot. This function has two variants:\n\n1. `list_mechanisms(slot_id)` - List mechanisms using a slot ID\n2. `list_mechanisms(slot)` - List mechanisms using a Slot struct\n\nThe mechanisms are returned as a list of atoms. If the mechanism is not known to P11ex\n(e.g. a vendor specific mechanism), it will be returned as an integer.","ref":"P11ex.Module.html#list_mechanisms/1"},{"type":"function","title":"P11ex.Module.list_slots/1","doc":"List all slots in the module. The `token_present?` argument is optional and\ndefaults to `true`. If set to `true`, only slots with a token present are returned.","ref":"P11ex.Module.html#list_slots/1"},{"type":"function","title":"P11ex.Module.login_type/0","doc":"Check if a successful login has been registered for a token in the\nPKCS#11 slot managed by this instance of `P11ex.Module`. The return value\nis `:user` or `:so` (security officer) if a login has been registered, or\n`nil` if no login has been registered.","ref":"P11ex.Module.html#login_type/0"},{"type":"function","title":"P11ex.Module.mechanism_info/2","doc":"Get information about a mechanism for a given slot. The mechanism is specified\nas an atom or an integer. For example, the mechanism `:ckm_aes_cbc` can also\nbe specified as the integer `0x00001082`:\n\n```elixir\n{:ok, info} = P11ex.Module.mechanism_info(slot, :ckm_aes_cbc)\n{:ok, info} = P11ex.Module.mechanism_info(slot, 0x00001082)\n```\n\nThe return value is a map with the following keys:\n\n* `flags` - The flags of the mechanism (a list of atoms, see `P11ex.Flags`). This\nindicates for what operations the mechanism can be used, e.g. `:encrypt`,\n`:decrypt`, `:sign`, `:verify`, etc.\n* `min_length` - The minimum key length supported by the mechanism (an integer)\n* `max_length` - The maximum key length supported by the mechanism (an integer)\n\nFor example, for `:ckm_aes_cbc` a typical return value is:\n\n```elixir\n%{flags: MapSet.new([:wrap, :encrypt, :decrypt]), min_length: 16, max_length: 32}\n```\n\nIf the mechanism is not known, the return value is\n`{:error, {:C_GetMechanismInfo, :ckr_mechanism_invalid}}`.","ref":"P11ex.Module.html#mechanism_info/2"},{"type":"function","title":"P11ex.Module.module_handle/0","doc":"Returns a reference to the handle of the PKCS#11 module. Usually, this\nis not needed by the application, but it can be useful if you need to\nperform operations on the module that are not otherwise provided by this\nlibrary.","ref":"P11ex.Module.html#module_handle/0"},{"type":"function","title":"P11ex.Module.open_session/2","doc":"","ref":"P11ex.Module.html#open_session/2"},{"type":"function","title":"P11ex.Module.register_login/1","doc":"Register a successful login for a token in the PKCS#11 slot managed\nby this instance of `P11ex.Module`. User type is `:user` or `:so`\n(security officer). If set, subsequent operations on the token will\nbe skipped. This is necessary to avoid login errors of value\n`:ckr_user_already_logged_in`. Can also be set to `nil` to unregister\na login.","ref":"P11ex.Module.html#register_login/1"},{"type":"function","title":"P11ex.Module.start_link/1","doc":"Start the `P11ex.Module` GenServer. The argument is the path to the\nPKCS#11 module file (shared library).","ref":"P11ex.Module.html#start_link/1"},{"type":"function","title":"P11ex.Module.token_info/1","doc":"Get information about a token in a slot. This function has two variants:\n\n1. `token_info(slot_id)` - Get token info using a slot ID\n2. `token_info(slot)` - Get token info using a Slot struct\n\nThe token information is based on the PKCS#11 structure `CK_TOKEN_INFO` and contains the following fields:\n\n* `label` - The label of the token (a string)\n* `manufacturer_id` - The manufacturer ID of the token (a string)\n* `model` - The model of the token (a string)\n* `serial_number` - The serial number of the token (a string)\n* `flags` - The flags of the token (a list of atoms, see `P11ex.Flags`)\n* `max_session_count` - The maximum number of sessions that can be opened for the token (an integer)\n* `session_count` - The number of sessions that are currently open for the token (an integer)\n* `max_rw_session_count` - The maximum number of read/write sessions that can be opened for the token (an integer)\n* `rw_session_count` - The number of read/write sessions that are currently open for the token (an integer)\n* `max_pin_len` - The maximum length of the PIN for the token (an integer)\n* `min_pin_len` - The minimum length of the PIN for the token (an integer)\n* `total_public_memory` - The total amount of public memory in the token (an integer)\n* `free_public_memory` - The amount of free public memory in the token (an integer)\n* `total_private_memory` - The total amount of private memory in the token (an integer)\n* `free_private_memory` - The amount of free private memory in the token (an integer)\n* `hardware_version` - The hardware version of the token (a tuple of integers)\n* `firmware_version` - The firmware version of the token (a tuple of integers)\n* `utc_time` - The UTC time of the token (a string)","ref":"P11ex.Module.html#token_info/1"},{"type":"module","title":"P11ex.Session","doc":"This module is a `GenServer` that manages a PKCS#11 session. A session is used\nto interact with a token, e.g. generate keys, encrypt data, decrypt data, etc. Sessions\nare created by the `P11ex.Module` module using the `open_session/3` function. Depending on\nthe type of token multiple for the same token can be opened in parallel (e.g. if the token is\na network HSM). One session can only be used in a serialised way, i.e. only one operation can be\nperformed at a time. Additionally, sessions have a state. This state can be non-persistent keys associated\nwith the session or the state of an encryption or decryption operation.\n\nTechnically, most PKCS#11 functions require to login to the token first using a PIN. That is, the\nlogin state is not connected to a particular session opened on a token. Many tokens raise an `:cka_already_logged_in`\nerror if a PIN is provided for a session that is already logged in. The `P11ex.Session` module tries to\nmake handling of the login state more easy by tracking the login state of the session. That is, only the\nfirst call to `login/3` will actually login to the token. Subsequent calls to `login/3` will check if\nthe session is already logged in and skip the login if so.","ref":"P11ex.Session.html"},{"type":"module","title":"Usage - P11ex.Session","doc":"The following examples show how to log into a token and create a new session.\n\n```elixir\n{:ok, module} = P11ex.Module.start_link(\"/usr/lib/softhsm/libsofthsm2.so\")\n{:ok, slot} = P11ex.Module.find_slot_by_tokenlabel(\"Token_0\")\n\n{:ok, session} = P11ex.Session.start_link(module: module, slot_id: slot.slot_id, flags: [:rw_session])\n:ok = P11ex.Session.login(session, :user, \"1234\")\n```","ref":"P11ex.Session.html#module-usage"},{"type":"function","title":"P11ex.Session.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"P11ex.Session.html#child_spec/1"},{"type":"function","title":"P11ex.Session.decrypt/4","doc":"Decrypt data using the specified `mechanism` and `key` in a single call. See\n`P11ex.Lib.encrypt_init/3` on how to select a decryption mechanism and\nset its parameters. Consider using `P11ex.Session.decrypt_init/3`,\n`P11ex.Session.decrypt_update/2`, and `P11ex.Session.decrypt_final/1` if you\nwant to decrypt data in chunks.","ref":"P11ex.Session.html#decrypt/4"},{"type":"function","title":"Example: Decrypt data in a single call - P11ex.Session.decrypt/4","doc":"```elixir\n{:ok, plaintext} = P11ex.Session.decrypt(session, {:ckm_aes_gcm, %{iv: iv, tag_bits: 128}}, key, ciphertext)\n```","ref":"P11ex.Session.html#decrypt/4-example-decrypt-data-in-a-single-call"},{"type":"function","title":"P11ex.Session.decrypt_final/1","doc":"Finalize the decryption operation that is in progress for this session\n(see `P11ex.Session.decrypt_init/3`).","ref":"P11ex.Session.html#decrypt_final/1"},{"type":"function","title":"P11ex.Session.decrypt_init/3","doc":"Initialize a decryption operation involving the specified `mechanism` and `key`. Many\nmechanisms require additional parameters. See `P11ex.Lib.encrypt_init/3` for more\ninformation on mechanisms and their parameters.\n\nThe function returns `:ok` if the operation is initialized successfully. The session\nis now in decryption mode and no other operations can be active at the same time. The\nciphertext can be provided in chunks using `P11ex.Session.decrypt_update/2` and\n`P11ex.Session.decrypt_final/1`.\n\nConsider using `P11ex.Session.decrypt/4` if you want to decrypt data in a single call.","ref":"P11ex.Session.html#decrypt_init/3"},{"type":"function","title":"Example: Decrypt data in chunks - P11ex.Session.decrypt_init/3","doc":"```elixir\n:ok = P11ex.Session.decrypt_init(session, {:ckm_aes_gcm, %{iv: iv, tag_bits: 128}})\n{:ok, plain_chunk1} = P11ex.Session.decrypt_update(session, cipher_chunk1)\n{:ok, plain_chunk2} = P11ex.Session.decrypt_update(session, cipher_chunk2)\n{:ok, plain_chunk3} = P11ex.Session.decrypt_final(session)\nplaintext = plain_chunk1 <> plain_chunk2 <> plain_chunk3\n```","ref":"P11ex.Session.html#decrypt_init/3-example-decrypt-data-in-chunks"},{"type":"function","title":"P11ex.Session.decrypt_update/2","doc":"Provide a chunk of ciphertext data to the decryption operation that is in\nprogress for this session (see `P11ex.Session.decrypt_init/3`).","ref":"P11ex.Session.html#decrypt_update/2"},{"type":"function","title":"P11ex.Session.destroy_object/2","doc":"Destroy the object specified by `object`.","ref":"P11ex.Session.html#destroy_object/2"},{"type":"function","title":"P11ex.Session.digest/2","doc":"Get the digest of the data provided to the digest operation. The session must be in the\n`:digest` state, so this function must be called after `digest_init/2`.","ref":"P11ex.Session.html#digest/2"},{"type":"function","title":"P11ex.Session.digest_final/1","doc":"Finalize the digest operation. The session must be in the `:digest` state,\nso this function must be called after `digest_init/2` and `digest_update/2`.\nIf the operation fails, the session's current operation is reset. The function\nreturns the digest.","ref":"P11ex.Session.html#digest_final/1"},{"type":"function","title":"P11ex.Session.digest_init/2","doc":"Initialize a digest operation involving the specified `mechanism`. The session's\ncurrent operation is set to `:digest`. This operation can be finalized by calling\n`digest_final/1` or `digest/1`. Also, a failure of `digest_update/2` will end\nthis state.","ref":"P11ex.Session.html#digest_init/2"},{"type":"function","title":"Example: Digest computation in chunks - P11ex.Session.digest_init/2","doc":"```elixir\n:ok = P11ex.Session.digest_init(session, {:ckm_sha256})\n:ok = P11ex.Session.digest_update(session, data1)\n:ok = P11ex.Session.digest_update(session, data2)\n{:ok, digest} = P11ex.Session.digest_final(session)\n```","ref":"P11ex.Session.html#digest_init/2-example-digest-computation-in-chunks"},{"type":"function","title":"Example: Digest computation in one go - P11ex.Session.digest_init/2","doc":"```elixir\n:ok = P11ex.Session.digest_init(session, {:ckm_sha256})\n{:ok, digest} = P11ex.Session.digest(session, data)\n```","ref":"P11ex.Session.html#digest_init/2-example-digest-computation-in-one-go"},{"type":"function","title":"P11ex.Session.digest_update/2","doc":"Provide data to the digest operation. The session must be in the `:digest` state,\nso this function must be called after `digest_init/2`. Call this function repeatedly\nwith chunks of data until all data has been provided. If the operation fails, the\nsession's current operation is reset.","ref":"P11ex.Session.html#digest_update/2"},{"type":"function","title":"P11ex.Session.encrypt/4","doc":"Encrypt data using the specified `mechanism` and `key` in a single call. See\n`P11ex.Lib.encrypt/4` on how to select an encryption mechanism and\nset its parameters.","ref":"P11ex.Session.html#encrypt/4"},{"type":"function","title":"Example: Encrypt data in a single call - P11ex.Session.encrypt/4","doc":"```elixir\niv = :crypto.strong_rand_bytes(12)\n{:ok, ciphertext} = P11ex.Session.encrypt(session, {:ckm_aes_gcm, %{iv: iv, tag_bits: 128}}, key, plaintext)\n```","ref":"P11ex.Session.html#encrypt/4-example-encrypt-data-in-a-single-call"},{"type":"function","title":"P11ex.Session.encrypt_final/1","doc":"Finalize the encryption operation that is in progress for this session.","ref":"P11ex.Session.html#encrypt_final/1"},{"type":"function","title":"P11ex.Session.encrypt_init/3","doc":"Initialize an encryption operation involving the specified `mechanism` and `key`.\nUse `P11ex.Session.encrypt_update/2` and `P11ex.Session.encrypt_final/1` to provide\nthe data to encrypt and produce the ciphertext chunks. Note that only one encryption\noperation can be active at a time for a given session. Consider using `P11ex.Session.encrypt/4`\nif you want to encrypt data in a single call and the data is not too large.\n\nThe function returns `:ok` if the operation is initialized successfully. That is, no\nother operations (e.g. decryption, signing, etc.) can be active at the same time.\n\nMany mechanisms require additional parameters. See `P11ex.Lib.encrypt_init/3` for more\ninformation on mechanisms and their parameters.","ref":"P11ex.Session.html#encrypt_init/3"},{"type":"function","title":"Example: Encrypt data in chunks - P11ex.Session.encrypt_init/3","doc":"```elixir\niv = :crypto.strong_rand_bytes(12)\n:ok = P11ex.Session.encrypt_init(session, {:ckm_aes_gcm, %{iv: iv, tag_bits: 128}})\n{:ok, cipher_chunk1} = P11ex.Session.encrypt_update(session, plain_chunk1)\n{:ok, cipher_chunk2} = P11ex.Session.encrypt_update(session, plain_chunk2)\n{:ok, cipher_chunk3} = P11ex.Session.encrypt_final(session)\nciphertext = cipher_chunk1 <> cipher_chunk2 <> cipher_chunk3\n```","ref":"P11ex.Session.html#encrypt_init/3-example-encrypt-data-in-chunks"},{"type":"function","title":"P11ex.Session.encrypt_update/2","doc":"Provide a chunk of plaintext data to the encryption operation that is in\nprogress for this session (see `P11ex.Session.encrypt_init/3`).","ref":"P11ex.Session.html#encrypt_update/2"},{"type":"function","title":"P11ex.Session.find_objects/3","doc":"Find objects in the session. The `attributes` is a list of tuples where the first\nelement is the attribute type and the second element is the value to match. The\n`max_hits` is the maximum number of hits to return. The result is a list of\n`P11ex.Lib.ObjectHandle.t()` objects.\n\nThis example shows how to find all secret keys in the session.\n\n```elixir\n  {:ok, objects} = P11ex.Session.find_objects(session, [{:cka_class, :cko_secret_key}], 10)\n```\n\nTo find all secret keys with the label \"my_key\" use the following:\n\n```elixir\n  {:ok, objects} = P11ex.Session.find_objects(session, [{:cka_class, :cko_secret_key}, {:cka_label, \"my_key\"}], 10)\n```","ref":"P11ex.Session.html#find_objects/3"},{"type":"function","title":"P11ex.Session.generate_key/3","doc":"Generate a symmetric key in the session. The key is generated according to the specified `mechanism`\nand the `key_template`. The `key_template` is a list of attributes that will be used to generate\nthe key. The function returns a handle to the generated key.\n\nSee `P11ex.Lib.generate_key/3` for more information on mechanisms and their parameters.","ref":"P11ex.Session.html#generate_key/3"},{"type":"function","title":"P11ex.Session.generate_key_pair/4","doc":"Generate a key pair in the session. The key pair is generated according to the specified `mechanism`\nand the `pub_key_template` and `priv_key_template`.\n\nSee `P11ex.Lib.generate_key_pair/4` for more information on mechanisms and their parameters.","ref":"P11ex.Session.html#generate_key_pair/4"},{"type":"function","title":"P11ex.Session.generate_random/2","doc":"Generate random data using the token's RNG.","ref":"P11ex.Session.html#generate_random/2"},{"type":"function","title":"P11ex.Session.info/1","doc":"Get information about the session. The result is a map with the following keys:\n* `:slot_id` - the slot ID of the session\n* `:state` - the state of the session\n* `:flags` - the flags of the session\n* `:device_error` - the device error of the session","ref":"P11ex.Session.html#info/1"},{"type":"function","title":"P11ex.Session.init/1","doc":"Initialize the session `GenServer`. This requires the `:module` (a `P11ex.Lib.ModuleHandle.t()`)\nand the `:slot_id` (an integer) of the slot the session is opened on. Additionally, the `:flags`\nkeyword argument can be used to pass additional flags to the `open_session/3` function.","ref":"P11ex.Session.html#init/1"},{"type":"function","title":"P11ex.Session.login/3","doc":"Log in to the session. The `user_type` must be either `:user` or `:so`. Provide the user's pin\nfor authentication. The `P11ex.Session` module checks if the session is already logged in and\nskips the login if so, preventing `:cka_already_logged_in` errors.","ref":"P11ex.Session.html#login/3"},{"type":"function","title":"P11ex.Session.logout/1","doc":"Logout from the session.","ref":"P11ex.Session.html#logout/1"},{"type":"function","title":"P11ex.Session.read_object/3","doc":"Read the attributes of the object identified by object handle `object`. The `type_hint`\nis an optional and can be used to specify the attributes to read. The default is to read\nthe common attributes (e.g. `:cka_class`, `:cka_id`). See `P11ex.Lib.ObjectAttributes`\nfor commonly used attribute sets.\n\nThe function returns a map of the successfully read attributes. The attributes that\ncould not be read (but were requested through the `type_hint`) are returned as the\nsecond element of the tuple. Reasons for not retrieving the attributes are that the\nattributes are not set or are sensitive.","ref":"P11ex.Session.html#read_object/3"},{"type":"function","title":"P11ex.Session.sign/2","doc":"Sign or MAC data. The session must be in the `:sign` state, so this function\nmust be called after `sign_init/3`. If the operation fails, the session's\ncurrent operation is reset. The function returns the signature or MAC.","ref":"P11ex.Session.html#sign/2"},{"type":"function","title":"P11ex.Session.sign_final/1","doc":"Finalize the signing operation or MAC computation. The session must\nbe in the `:sign` state, so this function must be called after\n`sign_init/3` and `sign_update/2`. If the operation fails, the session's\ncurrent operation is reset. The function returns the signature or MAC.","ref":"P11ex.Session.html#sign_final/1"},{"type":"function","title":"P11ex.Session.sign_init/3","doc":"Initialize a signing operation or MAC computation involving\nthe specified `mechanism` and `key`. The key type must be suitable for\nthe specified `mechanism`. If the initialization is successful, the\nsession's current operation is set to `:sign`. This operation can be\nfinalized by calling `sign_final/1` or `sign/2`. Also, a failure of\n`sign_update/2` will end this state.\n\nSee `P11ex.Lib.sign_init/3` for more information on mechanisms and their parameters.","ref":"P11ex.Session.html#sign_init/3"},{"type":"function","title":"P11ex.Session.sign_update/2","doc":"Provide data to the signing operation or MAC computation. The session must\nbe in the `:sign` state, so this function must be called after `sign_init/3`.\nCall this function repeatedly with chunks of data until all data has been\nprovided. If the operation fails, the session's current operation is reset.","ref":"P11ex.Session.html#sign_update/2"},{"type":"function","title":"P11ex.Session.start_link/1","doc":"","ref":"P11ex.Session.html#start_link/1"},{"type":"function","title":"P11ex.Session.verify/3","doc":"Verify a signature or MAC. The session must be in the `:verify` state, so this function\nmust be called after `verify_init/3`. If the operation fails, the session's current\noperation is reset.\n\nThe operation return `:ok` if the signature (or MAC) is valid. Otherwise, it returns\nan error. Typically, the error reason is `:ckr_signature_invalid` or `:ckr_signature_len_range`.","ref":"P11ex.Session.html#verify/3"},{"type":"function","title":"P11ex.Session.verify_init/3","doc":"Initialize a verification operation involving the specified `mechanism` and `key`.\nThe operation verifies signatures or MACs, depending the mechanism. Some mechanisms\nrequire additional parameters. See `P11ex.Lib.sign_init/3` for more information\non mechanisms and their parameters.\n\nIf successful, the session is in verification mode and no other operations can be\nactive at the same time.","ref":"P11ex.Session.html#verify_init/3"},{"type":"module","title":"P11ex.SessionPoolSupervisor","doc":"","ref":"P11ex.SessionPoolSupervisor.html"},{"type":"function","title":"P11ex.SessionPoolSupervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"P11ex.SessionPoolSupervisor.html#child_spec/1"},{"type":"function","title":"P11ex.SessionPoolSupervisor.init/1","doc":"Callback implementation for  Supervisor.init/1 .","ref":"P11ex.SessionPoolSupervisor.html#init/1"},{"type":"function","title":"P11ex.SessionPoolSupervisor.start_link/1","doc":"","ref":"P11ex.SessionPoolSupervisor.html#start_link/1"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}